import tdb
import sys
import time
import re
import string

vars=\
{           "full"             : 0,
            "debug"            : 0,
            "help"             : 0,
            "outer"            : "1",
            "inner"            : "1"
}
 
vars_doc=\
{
            "full"             :"Run more function call tests. NB: do not run two processes concurrently when\n      either one has this flag set",
            "debug"            :"Turn on debugging",
            "help"             :"Show option help",
            "outer"            :"Outer iterations",
            "inner"            :"Inner iterations",

}

optionexp=re.compile("^-{1,2}([^ \t\n\r\f\v=]+)\=?(\S+)?")

def print_usage():
  print("Usage:\n python test.py --<option>=<value>\n"+\
  "   or \n python test.py --<switch>\n")
  print("Switches:")
  keys=vars.keys()
  keys.sort()

  maxlen=0
  option_count=0
  for key in keys:
    if len(key)>maxlen: maxlen=len(key)
  for key in keys:
    if vars_doc.has_key(key):
      if type(vars[key])==str:
        option_count=option_count+1
        continue
      else:
        if vars[key]:
          value="True"
        else:
          value="False"
      print("%s : %s\n%s %s"%(string.rjust(key,maxlen),value,
            " "*maxlen,vars_doc[key]))
  if option_count>0:
    print("\nOptions:")
  for key in keys:
    if vars_doc.has_key(key):
      if type(vars[key])==str:
        value=vars[key]
      else:
        continue
      print("%s : %s\n%s %s"%(string.rjust(key,maxlen),value,
            " "*maxlen,vars_doc[key]))
        

for i in range(len(sys.argv)):
  res_o=optionexp.match(sys.argv[i])
  if not res_o: continue
  res=res_o.groups()
  try:
    if res[1]:
      if vars.has_key(res[0]):
        vars[res[0]]=res[1]
      else: 
        print "No such option '%s'"%res[0]
        raise ValueError("No such option '%s'"%res[0])
    else:
      if vars.has_key(res[0]):
        vars[res[0]]=1
      else:
        if string.find(res[0],'enable-')==0 and \
           vars.has_key(res[0][7:]):
           vars[res[0][7:]]=1
        elif string.find(res[0],'disable-')==0 and \
           vars.has_key(res[0][8:]):
           vars[res[0][8:]]=0
        else: raise ValueError("No such switch '%s'"%res[0])
  except:
    print_usage()
    sys.exit(0)

if vars['help']:
  print_usage()
  sys.exit(0)


debug=int(vars['debug'])

if vars["full"]:

  #----------------------------------------------------------
  db=tdb._tdb("test.db", tdb_flags=tdb.CLEAR_IF_FIRST)


  val="A value"
  db.store("A",val)
  print db.keys()  
  test_val=db.fetch("A")
  if val!=test_val:
    raise ValueError("Test value doesn't match")

  try:
    failed=1
    db.store("A",1)
  except TypeError:
    failed=0

  if failed: raise Exception("Store type checking failed")

  #----------------------------------------------------------
  print db.keys()
  for i in range(1000):
    s=str(i)
    db.store(s+"_key",s+"_value")

  key=db.firstkey()
  while key:
    val=db.fetch(key)
    key=db.nextkey(key)

  def traverse_func(key,val,extra):
    print key,val

  # Test traverse  
  db.traverse(traverse_func)

  # Test delete
  db.delete("A")

  # Test has_key
  if db.has_key("A"):
    raise KeyError("Key A not deleted")

  # Test keys()  
  keys=db.keys()
  l = len(keys)

  # Test popitem
  if l>0:
    item=db.popitem()
    if not item:
      raise Exception("popitem failed")

  # Test values
  keys=db.keys()
  values=db.values()
  if len(values)!=len(keys):
    raise Exception("values count does not match keys count")

  # Test items
  items=db.items()
  if len(items)!=len(keys):
    raise Exception("items count does not match keys count")

  # Test clear
  num=db.clear()
  if num!=len(keys):
    raise Exception("number of items cleared does not match keys count")

  # Test is_open()
  if not db.is_open():
   raise Exception("is_open() returned FALSE erroneously")
  

  db.close()


dct=tdb.tdb("test.db")

dct["0_key"]="0_value"
if not dct.has_key("0_key"):
  raise Exception("has_key operation failed")

ntrans=0
items=dct.items()  

outer=int(vars["outer"])
inner=int(vars["inner"])
starttime=time.time()
for j in range(outer):
  print j
  for i in range(inner):
    s=str(i)
    dct[s+"_key"]=s+"_value"

  for i in range(9000):
    item=dct.popitem()
    if item:
      try:
        del dct[item[0]]
      except KeyError:
        pass

  ntrans=ntrans+inner+18000

endtime=time.time()
print "Elapsed time is ",endtime-starttime,"s, ",ntrans/(endtime-starttime), " trans/s for ",ntrans," calls"
print "Finished OK"
